/**
 * @author Guillaume Santini
 * @version 1.0
 * Classe entitée permettant de décrire un devis.
 */
import java.util.HashSet;

public class Devis {
	private HashSet<Produit> panier ;
	private Discount discountStrategie ;
	private Taxe taxeStrategie ;
	private NumeroteurDeDevis numeroteur ;
	
	/**
	 * Constructeur par champs.
	 * @param numeroteur le numéroteur de devis
	 */
	public Devis(NumeroteurDeDevis numeroteur, HashSet<Produit> panier) {
		this.numeroteur = numeroteur;
		this.panier = panier;
	}
	
	/**
	 * Constructeur initialisé avec un panier vide.
	 * @param numeroteur le numéroteur de devis
	 */
	public Devis(NumeroteurDeDevis numeroteur) {
		this.numeroteur = numeroteur;
		this.panier = new HashSet<Produit>();
	}
	
	/**
	 * Setter permettant de fixer la stratégie de calcul de rabais.
	 * @param strategie la stratégie de calcul de rabais
	 */
	public void setStrategieDiscount(Discount strategie) {
		this.discountStrategie = strategie;
	}
	
	/**
	 * Setter permettnt de fixer la stratégie de calcul des taxes.
	 * @param strategie la stratégie de calcul des taxes
	 */
	public void setStrategieTaxe(Taxe strategie) {
		this.taxeStrategie = strategie;
	}
	
	/**
	 * Ajoute au contenu du panier un produit.
	 * @param produit le produit ajouté au panier
	 */
	public void ajouterAuPanier(Produit produit) {
		panier.add(produit);
	}
	
	/**
	 * Retire un produit du contenu du panier.
	 * @param produit le produit à retirer du panier
	 */
	public void retirerDuPanier(Produit produit) {
		panier.remove(produit);
	}
	
	/**
	 * Produit un devis correspondant au contenu du panier et affichant
	 * le montant HT du panier, le montant du rabais, le montant HT après
	 * application du rabais, le montant des taxes et le montant du panier
	 * TTC.
	 * @return le devis formaté
	 */
	public String editionDevis() {
		Double totalHT = 0. ;
		Double totalD  = 0. ;
		String facture = "--------------------------------------------------\n";
		facture += "Devis n° "+numeroteur.numeroDeDevis()+"\n";
		facture += "--------------------------------------------------\n";
		for (Produit p : panier) {
			facture += p+"\n";
			totalHT += p.getPrix() ;
			totalD  += this.discountStrategie.discount(p.getPrix()) ;
		}
		Double totalHTD = totalHT-totalD ;
		Double totalTVA = this.taxeStrategie.montantTaxe(totalHT-totalD) ;
		Double grandTotal = totalHTD + totalTVA ;
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Total HT",totalHT);
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Discount ("+this.discountStrategie+")",totalD);
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Total après discount",totalHTD);
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Taxes ("+this.taxeStrategie+")",totalTVA);
		facture += "--------------------------------------------------\n";
		facture += String.format("%-30s\t\t%10.2f\n","Total TTC",grandTotal);
		return facture;
	}
}
