
/**
 * 
 * @author Guillaume Santini
 * @version 1.0
 * Classe pour un singleton permettant de fournir des numéros de devis.
 * Les numéros fournis par l'instance sont uniques et incrémentés
 * à chaque fois qu'un numéro est demandé.
 * Le constructeur de cette classe est privé. Pour accéder à l'unique instance
 * il faut utiliser la méthode de classe polymorphe getInstance().
 * Lors de l'instanciation de la classe, le numéroteur est initialisé définitivement.
 * 
 */

public class NumeroteurDeDevis {
	private static Integer prochainNumero ;
	private static NumeroteurDeDevis instanceUnique;
	
	private NumeroteurDeDevis(Integer numeroInitial) {
		prochainNumero = numeroInitial;
	}
	
	/**
	 * Méthode de classe etournant l'instance unique de NumeroteurDeDevis.
	 * Cette méthode est à utiliser lors du premier appel car elle permet
	 * d'initialiser le numéroteur.
	 * Tout appel suivant à cette méthode produira un message d'alerte car le
	 * numéroteur n'est initialiser qu'une seule fois.
	 * @param numeroInitial
	 * @return l'instance unique de NumeroteurDeDevis
	 */
	public static NumeroteurDeDevis getInstance(Integer numeroInitial) {
		if ( instanceUnique == null ) {
			instanceUnique = new NumeroteurDeDevis(numeroInitial);
		} else {
			System.err.println("L'initialisation du numeroteur ne sera pas pris en compte car le numéroteur a déjà été initialisé.\n");
			System.err.println("Le prochain numero sera : "+ prochainNumero);
		}
		return instanceUnique;
	}
	
	/**
	 * Retourne l'instance unique de NumeroteurDeDevis.
	 * Cette méthode est à utiliser :
	 * - lors du premeir appel lorsque l'on veut initialiser le numéroteur
	 * avec la valeur par défaut 0, ou
	 * - lors des appels suivants lorsque l'on n'a plus besoin d'initialiser
	 * le numéroteur.
	 * @return l'instance unique de NumeroteurDeDevis
	 */
	public static NumeroteurDeDevis getInstance() {
		if ( instanceUnique == null ) {
			instanceUnique = new NumeroteurDeDevis(0);
		} 
		return instanceUnique;
	}
	
	/**
	 * Retourne un numéro de devis unique et incrémente le numéroteur.
	 * @return un numéro unique de devis
	 */
	public Integer numeroDeDevis() {
		return prochainNumero++;
	}
}
